"""This module contains the ``SeleniumMiddleware`` scrapy middleware"""

from importlib import import_module
from os.path import abspath, join, dirname

from scrapy.exceptions import NotConfigured
from selenium.webdriver.support.ui import WebDriverWait

from .http import SeleniumRequest, SeleniumResponse


class SeleniumMiddleware:
    """Scrapy middleware handling the requests using selenium"""

    _block_ads = False

    def __init__(self, driver_name, driver_executable_path, driver_arguments, browser_executable_path):
        """Initialize the selenium webdriver arguments

        Parameters
        ----------
        driver_name: str
            The selenium ``WebDriver`` to use
        driver_executable_path: str
            The path of the executable binary of the driver
        driver_arguments: list
            A list of arguments to initialize the driver
        browser_executable_path: str
            The path of the executable binary of the browser
        """

        webdriver_base_path = f'selenium.webdriver.{driver_name}'
        self.driver_name = driver_name

        driver_class_module = import_module(f'{webdriver_base_path}.webdriver')
        self.driver_class = getattr(driver_class_module, 'WebDriver')

        driver_options_module = import_module(f'{webdriver_base_path}.options')
        driver_options_class = getattr(driver_options_module, 'Options')

        driver_options = driver_options_class()
        if browser_executable_path:  # pragma: no cover
            driver_options.binary_location = browser_executable_path
        if '--block-ads' in driver_arguments:
            self._block_ads = True
            driver_arguments.remove('--block-ads')
        for argument in driver_arguments:
            driver_options.add_argument(argument)

        self.driver_kwargs = {'executable_path': driver_executable_path, 'options': driver_options}

    @classmethod
    def from_crawler(cls, crawler):
        """Initialize the middleware with the crawler settings"""

        driver_name = crawler.settings.get('SELENIUM_DRIVER_NAME')
        driver_executable_path = crawler.settings.get('SELENIUM_DRIVER_EXECUTABLE_PATH')
        browser_executable_path = crawler.settings.get('SELENIUM_BROWSER_EXECUTABLE_PATH')
        driver_arguments = crawler.settings.get('SELENIUM_DRIVER_ARGUMENTS')

        if not driver_name or not driver_executable_path:
            raise NotConfigured('SELENIUM_DRIVER_NAME and SELENIUM_DRIVER_EXECUTABLE_PATH must be set')

        middleware = cls(driver_name=driver_name, driver_executable_path=driver_executable_path,
                         driver_arguments=driver_arguments, browser_executable_path=browser_executable_path)

        return middleware

    def process_request(self, request, spider):
        """Process a request using the selenium driver if applicable"""

        if not isinstance(request, SeleniumRequest):
            return None

        driver = self.driver_class(**self.driver_kwargs)
        if self.driver_name == 'firefox' and self._block_ads:
            addon_path = join(dirname(abspath(__file__)), 'uBlock0@raymondhill.net.xpi')
            driver.install_addon(addon_path, temporary=True)
        driver.get(request.url)

        for cookie_name, cookie_value in request.cookies.items():
            driver.add_cookie({'name': cookie_name, 'value': cookie_value})

        if request.wait_until:
            WebDriverWait(driver, request.wait_time).until(request.wait_until)

        if request.screenshot:
            request.meta['screenshot'] = driver.get_screenshot_as_png()

        if request.script:
            driver.execute_script(request.script)

        body = str.encode(driver.page_source)

        # Expose the driver via the "meta" attribute
        request.meta.update({'driver': driver})

        return SeleniumResponse(driver.current_url, body=body, encoding='utf-8', request=request)

    def process_spider_output(self, response, result, spider):
        """Shutdown the driver when spider is finished processing the response"""
        response.interact.quit()
        return result



"""This module contains the packaging routine for oddsgym"""
from os import path
from setuptools import setup, find_packages
from configparser import ConfigParser


with open(path.join(path.abspath(path.dirname(__file__)), 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


def _get_version():
    with open(path.join(path.dirname(__file__), 'setup.cfg')) as setup_configuration_file:
        parser = ConfigParser()
        parser.read_string(setup_configuration_file.read())
        return parser['bumpversion']['current_version']


setup(packages=find_packages(exclude=["tests*"]),
      version=_get_version(),
      python_requires=">=3.6",
      long_description=long_description,
      long_description_content_type='text/markdown')



"""This module contains the ``SeleniumRequest`` class"""

from scrapy import Request
from scrapy.http import HtmlResponse
from parsel.csstranslator import HTMLTranslator
from cssselect.parser import SelectorSyntaxError
from selenium.webdriver.support.ui import WebDriverWait


class SeleniumRequest(Request):
    """Scrapy ``Request`` subclass providing additional arguments"""

    def __init__(self, wait_time=None, wait_until=None, screenshot=False, script=None, *args, **kwargs):
        """Initialize a new selenium request

        Parameters
        ----------
        wait_time: int
            The number of seconds to wait.
        wait_until: method
            One of the "selenium.webdriver.support.expected_conditions". The response
            will be returned until the given condition is fulfilled.
        screenshot: bool
            If True, a screenshot of the page will be taken and the data of the screenshot
            will be returned in the response "meta" attribute.
        script: str
            JavaScript code to execute.

        """

        self.wait_time = wait_time
        self.wait_until = wait_until
        self.screenshot = screenshot
        self.script = script

        super().__init__(*args, **kwargs)


class SeleniumResponse(HtmlResponse):

    @property
    def interact(self):
        """Shortcut to the WebDriver"""
        return self.meta['driver']

    def click(self, query):
        """Clicks on an element specified by query, and reloads the pages source into to the body"""
        try:
            xpath_query = HTMLTranslator().css_to_xpath(query)
        except SelectorSyntaxError:
            xpath_query = query
        self.interact.find_element_by_xpath(xpath_query).click()
        if self.request.wait_until:
            WebDriverWait(self.interact, self.request.wait_time).until(self.request.wait_until)
        return self.replace(url=self.interact.current_url,
                            body=str.encode(self.interact.page_source))
